#version 430
#extension GL_ARB_separate_shader_objects : enable





layout(location = 0) smooth in vec2 _fin_fragTexCoords;
layout(location = 1) flat in vec4 _fin_fragColor;
layout(location = 2) flat in uint _fin_fragDrawDim;
layout(location = 3) flat in uvec4 _fin_fragDrawDigitCounts;
layout(location = 4) flat in ivec4 _fin_fragValue;

struct SFragmentInput
{
	vec2 fragTexCoords;
	vec4 fragColor;
	uint fragDrawDim;
	uvec4 fragDrawDigitCounts;
	ivec4 fragValue;
	bool IsFrontFace;
};

layout(location = 0) out vec4 _fout_outColor;

struct SFragmentOutput
{
	vec4 outColor;
};

layout( std140) uniform USceneInfo
{
	mat4 ViewProj;
	mat4 View;
	mat4 Proj;
	mat4 InvView;
	mat4 InvViewProj;
	mat4 UserToLHZ;
	mat4 LHZToUser;
	mat4 BillboardingView;
	mat4 PackNormalView;
	mat4 UnpackNormalView;
	vec4 SideVector;
	vec4 DepthVector;
	vec2 ZBufferLimits;
	vec2 ViewportSize;
	float Handedness;
} SceneInfo;
 uniform sampler2D TextAtlas;
layout(std430 ) readonly buffer BAtlas
{
	uint Atlas[];
} Atlas;




void FragmentMain( in SFragmentInput  fInput,  out SFragmentOutput  fOutput  )
{



	const vec4 drawDigitSums = vec4( 0,
										fInput.fragDrawDigitCounts.x,
										fInput.fragDrawDigitCounts.x + fInput.fragDrawDigitCounts.y,
										fInput.fragDrawDigitCounts.x + fInput.fragDrawDigitCounts.y + fInput.fragDrawDigitCounts.z);
	const uint maxDigitCount = uint(drawDigitSums.w + fInput.fragDrawDigitCounts.w);
	const float globalDigitId = fInput.fragTexCoords.x * maxDigitCount;


	uint digitDim = fInput.fragDrawDim - 1;
	for (uint i = 0; i < fInput.fragDrawDim; ++i)
	{
		if (globalDigitId < drawDigitSums[i])
		{
			digitDim = i - 1;
			break;
		}
	}

	const float digitId = globalDigitId - drawDigitSums[digitDim];

	const uint digitCount = fInput.fragDrawDigitCounts[digitDim];
	const uint digitCountNoComma = digitCount - min(fInput.fragDrawDim - digitDim - 1U, 1U);



	const uint digitCountDecimal = digitCountNoComma - 1;


	const uint digitIdI = int(digitId);

	const vec4 digitIdI4 = vec4(digitIdI, digitIdI, digitIdI, digitIdI);
	const vec4 decimalSelector = vec4( equal(digitIdI4, vec4(digitCountDecimal - 3, digitCountDecimal - 2, digitCountDecimal - 1, digitCountDecimal - 0)) );
	const float decimalDigit = dot(1 / vec4(10000, 1000, 100, 10), decimalSelector);
	const vec2 fracSelector = vec2( equal(digitIdI4.xx, vec2(digitCountDecimal + 3, digitCountDecimal + 2)) );
	const float fracDigit = dot(1 * vec2(10, 1), fracSelector);
	const float digitFactor = 10 * (fracDigit + decimalDigit);

	const float absValue = abs(fInput.fragValue[digitDim]) + 0.0005f;


	const int digit = int(absValue * digitFactor) % 10;




	const int charDigit = digit + 48;
	const int charDecimalPoint = 46;
	const int charNegative = 45;
	const int charComma = 44;
	const int charEmpty = 0;

	const bool isCommaDigit = (digitIdI == digitCountNoComma);
	const bool isSignDigit = digitIdI == 0;




	const bool isDecimalPoint = false;



	const int charDigitOrComma = isCommaDigit ? charComma : charDigit;
	const int charInside = isDecimalPoint ? charDecimalPoint : charDigitOrComma;
	const int charOutside = (isSignDigit && (fInput.fragValue[digitDim] < 0)) ? charNegative : charEmpty;
	const int asciiCharCode = isSignDigit ? charOutside : charInside;



	const uint textureId = clamp(asciiCharCode - 32, 0, 126-32);


	const uint maxAtlasId =  Atlas.Atlas [ (0) ]  - 1U;
	const vec4 rect =  vec4( uintBitsToFloat(Atlas.Atlas [ (min(textureId, maxAtlasId) * 4 + 1) ]) , uintBitsToFloat(Atlas.Atlas [( (min(textureId, maxAtlasId) * 4 + 1) ) + 1]) , uintBitsToFloat(Atlas.Atlas [( (min(textureId, maxAtlasId) * 4 + 1) ) + 2]) , uintBitsToFloat(Atlas.Atlas [( (min(textureId, maxAtlasId) * 4 + 1) ) + 3]) ) ;
	const vec2 digitTexcoords = vec2(fract(digitId), fInput.fragTexCoords.y);
	const vec2 texcoords = digitTexcoords * rect.xy + rect.zw;

	fOutput.outColor =  texture(TextAtlas, texcoords) ;


	if (fOutput.outColor.r < 0.1f)
		discard;

	fOutput.outColor *= fInput.fragColor;
}


void main()
{
	SFragmentInput fInput;
	SFragmentOutput fOutput;

	fInput.fragTexCoords = _fin_fragTexCoords;
	fInput.fragColor = _fin_fragColor;
	fInput.fragDrawDim = _fin_fragDrawDim;
	fInput.fragDrawDigitCounts = _fin_fragDrawDigitCounts;
	fInput.fragValue = _fin_fragValue;
	fInput.IsFrontFace = gl_FrontFacing;
	FragmentMain(fInput, fOutput);
	_fout_outColor = fOutput.outColor;
}
